@@ -581,7 +581,7 @@ int plextor (Scsi_Command &cmd,int action,int book,int gen=2)
 		if ((err=cmd.transport(READ,vector,sizeof(vector))))
 		    return sperror ("PLEXTOR_E9h(A)",err),1;
 
-		printf ("Unit was instruction to brand DVD+R media as DVD%s\n",
+		printf ("Unit was instructed to brand DVD+R media as DVD%s\n",
 			book&0xF0?"+R":"-ROM");
 
 		if (gen<3)	return 0;
@@ -596,7 +596,7 @@ int plextor (Scsi_Command &cmd,int action,int book,int gen=2)
 		if ((err=cmd.transport(READ,vector,sizeof(vector))))
 		    return sperror ("PLEXTOR_E9h(E)",err),1;
 
-		printf ("Unit was instruction to brand DVD+R DL media as DVD%s\n",
+		printf ("Unit was instructed to brand DVD+R DL media as DVD%s\n",
 			book&0xF0?"+R DL":"-ROM");
 
 		return 0;
@@ -653,7 +653,7 @@ int lg (Scsi_Command &cmd,int action,int book)
 		if ((err=cmd.transport(WRITE,vector,sizeof(vector))))
 		    return sperror ("LG_FCh",err),1;
 
-		printf ("Unit was instruction to brand DVD+R media as DVD%s\n",
+		printf ("Unit was instructed to brand DVD+R media as DVD%s\n",
 			book&0xF0?"+R":"-ROM");
 		return 0;
 		break;
