@@ -146,6 +146,21 @@ int media_reload (char *name=NULL,struct stat *sb=NULL,unsigned int cap2k=0)
 		execl("/usr/bin/volrmmount","volrmmount","-i",name,(void*)NULL);
 		return 0;	// not normally reached
 	    }
+	    else if (1)
+	    {	char srlink[256],*dsk;
+		static char dev[]="/dev/dsk/";
+		int len;
+
+		/* this basically handles /dev/rsrN link */
+		if ((len=readlink(name,srlink,sizeof(srlink)-1))>0 &&
+		    (srlink[len]='\0',(dsk=strstr(srlink,"dsk/"))!=NULL))
+		{   name=(char *)alloca(sizeof(dev)+strlen(dsk+4)+1);
+		    strcpy(name,dev), strcat(name,dsk+4);
+		}
+		setuid(getuid());
+		execl("/usr/bin/rmmount","rmmount",name,(void*)NULL);
+		return 0;	// not reached on SunOS 5.11
+	    }
 #endif
 	    else return 0;	// m-m-m-m! patched kernel:-)
 	}
@@ -433,18 +448,19 @@ extern "C"
 float get_buffer_stats (void *fd)
 { Scsi_Command		cmd(fd);
   unsigned char		bcap[12];
-  unsigned int		bsize,bfree,err;
+  unsigned int		bsize,bfree;
+  int			err;
 
     cmd[0] = 0x5C;              // READ BUFFER CAPACITY
     cmd[8] = sizeof(bcap);
     cmd[9] = 0;
     if ((err=cmd.transport (READ,bcap,sizeof(bcap))))
-	return -err;
+	return -1.0f;
 
     bsize = bcap[4]<<24|bcap[5]<<16|bcap[6]<<8|bcap[7];
     bfree = bcap[8]<<24|bcap[9]<<16|bcap[10]<<8|bcap[11];
 
-  return bsize ? (1.0 - (float)bfree/(float)bsize) : 0.0;
+  return bsize ? (1.0f - (float)bfree/(float)bsize) : 0.0f;
 }
 
 ssize_t poor_mans_pwrite64 (int fd,const void *_buff,size_t size,off64_t foff)
@@ -458,8 +474,8 @@ ssize_t poor_mans_pwrite64 (int fd,const void *_buff,size_t size,off64_t foff)
     if (foff&0x7FFF || size&0x7FFF)	// 32K block size
 	return (errno=EINVAL,-1);
 
-    lba = foff>>11;
-    nbl = size>>11;
+    lba = (unsigned int)(foff/2048);
+    nbl = (unsigned int)(size/2048);
 
     if (!media_written && next_wr_addr)
     {	if ((lba+nbl) <= next_wr_addr)
@@ -843,7 +859,7 @@ static int plus_rw_restart_format (Scsi_Command &cmd, off64_t size)
   int err,i,len;
 
     if (!dvd_compat && size!=0)
-    {	blocks = size/2048;
+    {	blocks = (unsigned int)(size/2048);
 	blocks += 15, blocks &= ~15;
 
 	lead_out = 0;
@@ -1490,9 +1506,10 @@ opc:
 		break;
 	    }
 	    sperror ("PERFORM OPC",err);
-	    if ((err&0x0FF00)==0x07300)	exit(FATAL_START(errno));
+	    if ((err&0x0FF00)==0x07300 && (err&0xF0000)!=0x10000)
+		exit(FATAL_START(errno));
 	    /*
-	     * Th rest of errors are ignored, most notably observed:
+	     * The rest of errors are ignored, most notably observed:
 	     * 0x52000, "INVALID COMMAND"		Panasonic LD-F321
 	     * 0x52700, "WRITE PROTECTED"		Plextor PX-712A
 	     * 0x52C00, "COMMAND SEQUENCE ERROR"	Plextor PX-716UF
@@ -1514,7 +1531,7 @@ static int minus_rw_quickgrow (Scsi_Command &cmd,off64_t size)
     type = formats[4+4]&3;
 
     if (type==2 && size!=0)
-    {	blocks = size/2048;
+    {	blocks = (unsigned int)(size/2048);
 	blocks += 15, blocks &= ~15;
 
 	lead_out = 0;
@@ -1570,7 +1587,7 @@ static int minus_r_reserve_track (Scsi_Command &cmd,off64_t size)
 { int          err;
   unsigned int blocks;
 
-    blocks = size/2048;
+    blocks = (unsigned int)(size/2048);
     if (is_dao)	dao_blocks = blocks;
     else	blocks += 15, blocks &= ~15;
 
@@ -1619,7 +1636,7 @@ static void plus_r_dl_split (Scsi_Command &cmd,off64_t size)
 
     split = dvd_20[8]<<24|dvd_20[9]<<16|dvd_20[10]<<8|dvd_20[11];
 
-    blocks = size/2048;
+    blocks = (unsigned int)(size/2048);
     blocks += 15, blocks &= ~15;
 
     if (blocks <= split)
@@ -1692,14 +1709,16 @@ void no_r_finalize ()
 	pioneer_stop (cmd);
 
 	media_written = 0;
-	errno = 0;
+	errno = EINTR;
     }
     _exit (errno);
 }
 
 extern "C"
 void plus_rw_finalize ()
-{   sigs_mask(SIGS_BLOCK);
+{ int saved_errno = errno;
+
+    sigs_mask(SIGS_BLOCK);
     while (media_written)
     { Scsi_Command cmd(ioctl_handle);
       int err;
@@ -1731,17 +1750,20 @@ void plus_rw_finalize ()
  
 	media_written = 0;
 	next_wr_addr = 0;
-	errno = 0;
 
 	if (do_reload && media_reload())
 	    perror (":-( unable to reload tray");// not actually reached
+
+	errno = saved_errno;
     }
     _exit (errno);
 }
 
 extern "C"
 void bd_re_finalize ()
-{   sigs_mask(SIGS_BLOCK);
+{ int saved_errno = errno;
+
+    sigs_mask(SIGS_BLOCK);
     while (media_written)
     { Scsi_Command cmd(ioctl_handle);
 
@@ -1750,17 +1772,20 @@ void bd_re_finalize ()
 
 	media_written = 0;
 	next_wr_addr = 0;
-	errno = 0;
 
 	if (do_reload && media_reload())
 	    perror (":-( unable to reload tray");// not actually reached
+
+	errno = saved_errno;
     }
     _exit (errno);
 }
 
 extern "C"
 void plus_r_finalize ()
-{   sigs_mask(SIGS_BLOCK);
+{ int saved_errno = errno;
+
+    sigs_mask(SIGS_BLOCK);
     while (media_written)
     { Scsi_Command cmd(ioctl_handle);
       int          mode,err;
@@ -1776,7 +1801,7 @@ void plus_r_finalize ()
 	cmd[5] = next_track;
 	cmd[9] = 0;
 	if ((err=cmd.transport()))
-	    sperror ("CLOSE TRACK",err);
+	    sperror ("CLOSE TRACK",err), saved_errno = errno;
 
 	if (wait_for_unit (cmd)) break;
 
@@ -1800,24 +1825,28 @@ void plus_r_finalize ()
 		continue;
 	    }
 	    else if (err)
-		sperror (dvd_compat?"CLOSE DISC":"CLOSE SESSION",err);
+		sperror (dvd_compat?"CLOSE DISC":"CLOSE SESSION",err),
+		saved_errno = errno;
 	    break;
 	}
 
 	if (wait_for_unit (cmd)) break;
  
 	media_written = 0;
-	errno = 0;
 
 	if (do_reload && media_reload())
 	    perror (":-( unable to reload tray");// not actually reached
+
+	errno = saved_errno;
     }
     _exit (errno);
 }
 
 extern "C"
 void minus_rw_finalize ()
-{   sigs_mask(SIGS_BLOCK);
+{ int saved_errno = errno;
+
+    sigs_mask(SIGS_BLOCK);
     while (media_written)
     { Scsi_Command cmd(ioctl_handle);
       int err;
@@ -1832,7 +1861,7 @@ void minus_rw_finalize ()
 	    cmd[2] = 0x02;		// "Close Session"
 	    cmd[9] = 0;
 	    if ((err=cmd.transport()))
-		sperror ("CLOSE SESSION",err);
+		sperror ("CLOSE SESSION",err), saved_errno = errno;
 
 	    if (wait_for_unit (cmd)) break;
 
@@ -1841,20 +1870,23 @@ void minus_rw_finalize ()
 
 	media_written = 0;
 	next_wr_addr = 0;
-	errno = 0;
 
 	if (do_reload)
 	{   if (media_reload())
 		perror (":-( unable to reload tray");// not actually reached
 	}
 	else return;	// Restricted Overwrite is just ugly!
+
+	errno = saved_errno;
     }
     _exit (errno);
 }
 
 extern "C"
 void minus_r_finalize ()
-{   sigs_mask(SIGS_BLOCK);
+{ int saved_errno = errno;
+
+    sigs_mask(SIGS_BLOCK);
     while (media_written)
     { Scsi_Command cmd(ioctl_handle);
       int err;
@@ -1871,7 +1903,7 @@ void minus_r_finalize ()
 	    cmd[5] = next_track;
 	    cmd[9] = 0;
 	    if ((err=cmd.transport()))
-		sperror ("CLOSE TRACK",err);
+		sperror ("CLOSE TRACK",err), saved_errno = errno;
 
 	    if (wait_for_unit (cmd)) break;
 
@@ -1889,7 +1921,8 @@ void minus_r_finalize ()
 		    continue;
 		}
 		else if (err)
-		    sperror (dvd_compat?"CLOSE DISC":"CLOSE SESSION",err);
+		    sperror (dvd_compat?"CLOSE DISC":"CLOSE SESSION",err),
+		    saved_errno = errno;
 		break;
 	    }
 
@@ -1897,17 +1930,20 @@ void minus_r_finalize ()
 	}
  
 	media_written = 0;
-	errno = 0;
 
 	if (do_reload && media_reload())
 	    perror (":-( unable to reload tray");// not actually reached
+
+	errno = saved_errno;
     }
     _exit (errno);
 }
 
 extern "C"
 void bd_r_finalize ()
-{   sigs_mask(SIGS_BLOCK);
+{ int saved_errno = errno;
+
+    sigs_mask(SIGS_BLOCK);
     while (media_written)
     { Scsi_Command  cmd(ioctl_handle);
       int           err;
@@ -1923,7 +1959,7 @@ void bd_r_finalize ()
 	cmd[5] = next_track;
 	cmd[9] = 0;
 	if ((err=cmd.transport()))
-	    sperror ("CLOSE TRACK",err);
+	    sperror ("CLOSE TRACK",err), saved_errno = errno;
 
 	if (wait_for_unit (cmd)) break;
 
@@ -1940,7 +1976,7 @@ void bd_r_finalize ()
 		    continue;
 		}
 		else if (err)
-		    sperror ("CLOSE SESSION",err);
+		    sperror ("CLOSE SESSION",err), saved_errno = errno;
 		break;
 	    }
 
@@ -1948,19 +1984,21 @@ void bd_r_finalize ()
  	}
  
 	media_written = 0;
-	errno = 0;
 
 	if (do_reload && media_reload())
 	    perror (":-( unable to reload tray");// not actually reached
+
+	errno = saved_errno;
     }
     _exit (errno);
 }
 
 extern "C"
 void ram_reload ()
-{   sigs_mask(SIGS_BLOCK);
+{ int saved_errno = errno;
+    sigs_mask(SIGS_BLOCK);
     if (media_written && do_reload) media_reload();
-    _exit (errno);
+    _exit (saved_errno);
 }
 
 //
@@ -2045,7 +2083,7 @@ pwrite64_t poor_mans_setup (void *fd,off64_t leadout)
 	    plusminus_pages_setup (cmd,profile);
 	    if ((profile==0x15) && leadout)
 	    {	unsigned char dvd_10[4+16];
-		unsigned int layer_size,data_size=leadout/2048;
+		unsigned int layer_size,data_size=(unsigned int)(leadout/2048);
 
 		cmd[0] = 0xAD;	// READ DVD STRUCTURE
 		cmd[7] = 0x10;
