@@ -31,15 +31,21 @@ static void  __semaphore_close(void *h)	{ CloseHandle(h); }
 #include <sched.h>
 #include <semaphore.h>
 #include <stdlib.h>
+#include <limits.h>
 
 #define THR_TYPE int
 
 static void *__thread_create(int (*func)(void *),void *arg)
 { pthread_t h;
   pthread_attr_t attr;
+  size_t stack_sz=16*1024;
+
+#ifdef PTHREAD_STACK_MIN
+    stack_sz += PTHREAD_STACK_MIN;
+#endif
 
     if (pthread_attr_init(&attr)==0 &&
-	pthread_attr_setstacksize(&attr,64*1024)==0 &&
+	pthread_attr_setstacksize(&attr,stack_sz)==0 &&
 	pthread_attr_setscope(&attr,PTHREAD_SCOPE_SYSTEM)==0 &&
 	pthread_create(&h,&attr,(void *(*)(void *))func,arg)==0 )
 	return (void *)h;
@@ -55,7 +61,7 @@ static int   __thread_wait(void *h)
 }
 
 #if defined(__APPLE__) && defined(__MACH__)
-// Mac OS X doesn't implement unnamed semphores, g-r-r-r-r...
+// Mac OS X doesn't implement unnamed semaphores, g-r-r-r-r...
 static void *__semaphore_create(int max)
 { sem_t *h = sem_open ("growisofs",O_CREAT|O_EXCL,0700,0);
     if (h==(sem_t*)SEM_FAILED)	return NULL;
