@@ -10,6 +10,7 @@
 # define EBUSY		ERROR_SHARING_VIOLATION
 # define ENOENT		ERROR_FILE_NOT_FOUND
 # define ENOTDIR	ERROR_NO_VOLUME_LABEL
+# define EINTR		ERROR_OPERATION_ABORTED
 # define FATAL_START(e)	(0x10000|(e))
 # define FATAL_MASK	 0x0FFFF
 
@@ -35,23 +36,23 @@ static class __win32_errno__ {
 #endif
 #define perror win32_perror
 static void win32_perror (const char *str)
-{ LPVOID lpMsgBuf;
+{ LPSTR lpMsgBuf;
 
-    FormatMessage( 
+    FormatMessageA( 
 	FORMAT_MESSAGE_ALLOCATE_BUFFER |
 	FORMAT_MESSAGE_FROM_SYSTEM | 
 	FORMAT_MESSAGE_IGNORE_INSERTS,
 	NULL,
 	GetLastError(),
 	0, // Default language
-	(LPTSTR) &lpMsgBuf,
+	(LPSTR) &lpMsgBuf,
 	0,
 	NULL 
 	);
     if (str)
-	fprintf (stderr,"%s: %s",str,(char *)lpMsgBuf);
+	fprintf (stderr,"%s: %s",str,lpMsgBuf);
     else
-	fprintf (stderr,"%s",(char *)lpMsgBuf);
+	fprintf (stderr,"%s",lpMsgBuf);
 
     LocalFree(lpMsgBuf);
 }
